/** \file
    \author Tomasz Ostrowski
    \date Q1 2006
	Terminal USB - przykad wykorzystania linkowanej statycznie biblioteki libusb.
    Pliki libusb.lib oraz usb.h pochodz z projektu libusb.

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <inifiles.hpp>
#include "mainUnit.h"
#include "common.h"
#include "TCommThread.h"
#include "about.h"
#include "SettingsGeneralUnit.h"
#include "usb.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
#pragma link "libusb.lib"

TmainForm *mainForm;
TCommThread *CommThread;

struct usb_device *my_dev;

//zmienne importowane z common.cpp
extern struct usb_dev_handle *usb_handle;
extern int VENDOR_ID;
extern int PRODUCT_ID;
extern int EP_OUT;
extern int EP_IN;
extern int transfer_type;          //bulk or interrupt ( 1 or 2 value )              

void __fastcall Log(AnsiString tekst)
{
    //logowanie zdarzen wszelakich
    //TODO: kolorowanie!!!
    mainForm->RichEdit1->SelAttributes->Color = clGray;
    mainForm->RichEdit1->SelAttributes->Height += 5;
    mainForm->RichEdit1->Text = mainForm->RichEdit1->Text + tekst;
    //TODO: nie ma tu ograniczen na dlugosc tekstu?
    mainForm->RichEdit1->SelStart = mainForm->RichEdit1->GetTextLen(); // Unless it's already there
    mainForm->RichEdit1->SelLength=0;
    ::SendMessage(mainForm->RichEdit1->Handle, EM_SCROLLCARET, 0, 0);
}

//USB-related
static struct usb_device *device_init(void)
{
    struct usb_bus *usb_bus;
    struct usb_device *dev;
    usb_init();
    Log("USB : Enumerating USB busses and devices...\r\n");
    usb_find_busses();
    usb_find_devices();
    for (usb_bus = usb_get_busses(); usb_bus;
         usb_bus = usb_bus->next) {
            for (dev = usb_bus->devices; dev; dev = dev->next)
                {
                if (dev->descriptor.idVendor == VENDOR_ID)
                    {
                    //Log("Device with specified VID found\r\n");
                    if(dev->descriptor.idProduct == PRODUCT_ID)
                        {
                        //Log("Device with specified PID found\r\n");
                        Log("Device found\r\n");
                        return dev;
                        }
                    }
                }
    }
    return NULL;
}


//---------------------------------------------------------------------------
__fastcall TmainForm::TmainForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TmainForm::FormCreate(TObject *Sender)
{
char txtbuf[300];
//inicjalizacja - plik ini o tej samej nazwie jak plik exe
TIniFile *ini;
ini = new TIniFile( ChangeFileExt( Application->ExeName, ".ini" ) );
sscanf( ini->ReadString("USB","VID","0x04D8").c_str(), "%i", &VENDOR_ID );
sprintf(txtbuf,"0x%04X",VENDOR_ID);    //ponowne sformatowanie
VIDedit->Text = txtbuf;
sscanf( ini->ReadString("USB","PID","0x00AA").c_str(), "%i", &PRODUCT_ID );
sprintf(txtbuf,"0x%04X",PRODUCT_ID);    //ponowne sformatowanie
PIDedit->Text = txtbuf;
sscanf( ini->ReadString("USB","EPout","0x03").c_str(), "%i", &EP_OUT );
sprintf(txtbuf,"0x%02X",EP_OUT);    //ponowne sformatowanie
EPOUTedit->Text = txtbuf;
sscanf( ini->ReadString("USB","EPin","0x83").c_str(), "%i", &EP_IN );
sprintf(txtbuf,"0x%02X",EP_IN);    //ponowne sformatowanie
EPINedit->Text = txtbuf;
if( ini->ReadString("USB","Transfer Type","BULK") == "BULK" ) transfer_type=1;
else transfer_type=2;
mainForm->ComboBox1->ItemIndex = transfer_type - 1;
settings.bRecvMode = ini->ReadInteger("Receiving","Mode",0);
delete ini;

//proba skomunikowania z urzadzeniem USB
int ret = 1;
int i;
my_dev = device_init();
if (my_dev == NULL) {
    Log("device not found!\r\n");
}
else
    {
    //open device
    usb_handle = usb_open( my_dev );
    if(usb_handle == NULL)
                    Application->MessageBoxA("device opening failed","",NULL);
    else
        {
        Log("Device opened\r\n");
        //claim interface
        //usb_claim_interface(usb_dev_handle *dev, int interface);

        //set configuration
        if ( ( ret = usb_set_configuration( usb_handle, 1 ) ) < 0 )
            {
            sprintf(txtbuf,"config set failed: %s\r\n", usb_strerror());
            Log( txtbuf );
            }
        else
            {
            Log("config set ok\r\n");
            }

        Log("Claiming interface...");
        ret = usb_claim_interface( usb_handle, 1 );   //chyba interfejs 1 - ten od BULK
        if( ret < 0 )
            {
            sprintf(txtbuf,"FAIL: %s\r\n", usb_strerror());
            Log( txtbuf );
            }
        else
            {
            Log( "OK\r\n" );
            ConnectBtn->Caption = "Reconnect";
            CommThread = new TCommThread(false);
            }
        }
    }

}
//---------------------------------------------------------------------------
void __fastcall TmainForm::Edit1KeyPress(TObject *Sender, char &Key)
{
int ret;
char txtbuf[300];
if(usb_handle!=NULL && Key==13) //Enter
    {
    if(transfer_type == 1)
        ret = usb_bulk_write( usb_handle, EP_OUT, Edit1->Text.c_str(), Edit1->Text.Length(), 2000 );  //0x01?
    else
        {
        ret = usb_interrupt_write( usb_handle, EP_OUT, Edit1->Text.c_str(), Edit1->Text.Length(), 2000 );  //0x01?
        //Log("int write ");
        }
    if(ret<0)
        {
        sprintf(txtbuf,"USB write failed: %s\r\n", usb_strerror());
        Log( txtbuf );
        }
    else
        {
        sprintf(txtbuf,"\r\nSent: %s\r\n", Edit1->Text.c_str());
        Log( txtbuf );
        }
    Edit1->Text = "";
    }
}
//---------------------------------------------------------------------------
void __fastcall TmainForm::ConnectBtnClick(TObject *Sender)
{
int ret, temp;
char txtbuf[300];
bool valid=true;

if(usb_handle!=NULL)
    {
    CommThread->Terminate();
    Sleep(500); //wait for terminate
    //release usb device
    Log( "Releasing device... " );
    if(usb_handle!=NULL)
        {
        if ( ( ret = usb_release_interface( usb_handle, 1 ) ) < 0 )
            {
            sprintf(txtbuf,"FAILED: %s\r\n", usb_strerror());
            Log( txtbuf );
            }
        else
            {
            Log( "OK\r\n" );
            usb_close(usb_handle);
            }
        }
    }

    Log( "Reconecting...\r\n" );

    if( sscanf( VIDedit->Text.c_str(), "%i", &temp )>0 && temp<=65535 & temp>=0)
        VENDOR_ID = temp;
    else
        {
        Application->MessageBoxA("VID value incorrect or out of range", "Settings error", NULL);
        valid=false;
        }
    if( sscanf( PIDedit->Text.c_str(), "%i", &temp )>0 && temp<=65535 & temp>=0)
        PRODUCT_ID = temp;
    else
        {
        Application->MessageBoxA("PID value incorrect or out of range", "Settings error", NULL);
        valid=false;
        }
    if( sscanf( PIDedit->Text.c_str(), "%i", &temp )>0 && temp<=65535 & temp>=0)
        PRODUCT_ID = temp;
    else
        {
        Application->MessageBoxA("PID value incorrect or out of range", "Settings error", NULL);
        valid=false;
        }
    if( sscanf( EPOUTedit->Text.c_str(), "%i", &temp )>0 && temp<=255 & temp>=0)
        EP_OUT = temp;
    else
        {
        Application->MessageBoxA("EP_OUT value incorrect or out of range", "Settings error", NULL);
        valid=false;
        }
    if( sscanf( EPINedit->Text.c_str(), "%i", &temp )>0 && temp<=255 & temp>=0)
        EP_IN = temp;
    else
        {
        Application->MessageBoxA("EP_IN value incorrect or out of range", "Settings error", NULL);
        valid=false;
        }
    if(valid)
        {
        transfer_type = mainForm->ComboBox1->ItemIndex + 1;
        my_dev = device_init();
        if (my_dev == NULL)
            {
            Log("device not found!\r\n");
            }
        else
            {
            //open device
            usb_handle = usb_open( my_dev );
            if(usb_handle == NULL)
                            Application->MessageBoxA("device opening failed","",NULL);
            else
                {
                Log("Device opened\r\n");
                //claim interface
                //usb_claim_interface(usb_dev_handle *dev, int interface);

                //set configuration
                usb_set_configuration( usb_handle, 1 );
                if( ret < 0 )
                    {
                    sprintf(txtbuf,"config set failed: %s\r\n", usb_strerror());
                    Log( txtbuf );
                    }
                else
                    {
                    Log("config set ok\r\n");
                    }

                Log("Claiming interface...");
                if ( ( ret = usb_claim_interface( usb_handle, 1 ) ) < 0 )   //chyba interfejs 1 - ten od BULK
                    {
                    sprintf(txtbuf,"FAIL: %s\r\n", usb_strerror());
                    Log( txtbuf );
                    }
                else
                    {
                    Log( "OK\r\n" );
                    ConnectBtn->Caption = "Reconnect";
                    CommThread = new TCommThread(false);
                    }
                }
            }
        }//end of if(valid)

}
//---------------------------------------------------------------------------

void __fastcall TmainForm::FormDestroy(TObject *Sender)
{
char txtbuf[300];
int ret;
//release usb device
Log( "RELEASE DEVICE: " );
if(usb_handle!=NULL)
    {
    ret = usb_release_interface( usb_handle, 1 );
    if(ret<0)
        {
        sprintf(txtbuf,"FAILED: %s\r\n", usb_strerror());
        Log( txtbuf );
        }
    else
        Log( "OK\r\n" );
    usb_close(usb_handle);
    }
//zapis do pliku ini
TIniFile *ini;
ini = new TIniFile(ChangeFileExt( Application->ExeName, ".ini" ) );
sprintf(txtbuf,"0x%04X",VENDOR_ID);
ini->WriteString("USB","VID",txtbuf);
sprintf(txtbuf,"0x%04X",PRODUCT_ID);
ini->WriteString("USB","PID",txtbuf);
sprintf(txtbuf,"0x%02X",EP_OUT);
ini->WriteString("USB","EPout",txtbuf);
sprintf(txtbuf,"0x%02X",EP_IN);
ini->WriteString("USB","EPin",txtbuf);
if(transfer_type == 1) ini->WriteString("USB","Transfer Type","BULK");
if(transfer_type == 2) ini->WriteString("USB","Transfer Type","INTERRUPT");
ini->WriteInteger("Receiving","Mode",settings.bRecvMode);
delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TmainForm::Exit1Click(TObject *Sender)
{
	this->Close();
}
//---------------------------------------------------------------------------

void __fastcall TmainForm::About1Click(TObject *Sender)
{
	AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TmainForm::General1Click(TObject *Sender)
{
	SettingsGeneral->ShowModal();
}
//---------------------------------------------------------------------------

